//
//  SealedPad.m
//  SealedPad
//
//  Created by 柴田 正明 on 10/09/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "Cryptograph.h"
#import "MyDocument.h"
#import "PreferenceController.h"

@implementation MyDocument

@synthesize encryptedData;
@synthesize passwordString;

- (id)init
{
    self = [super init];
    if (self) {
        self.encryptedData = nil;
        self.passwordString = nil;
    }
    return self;
}

- (void)dealloc
{
    [encryptedData release];
    [passwordString release];
    [super dealloc];
}

- (id)initWithContentsOfURL:(NSURL *)absoluteURL ofType:(NSString *)typeName error:(NSError **)outError
{
    return [super initWithContentsOfURL:absoluteURL ofType:typeName error:outError];
}

- (NSString *)windowNibName
{
    return @"SealedPad";
}

- (void)windowControllerDidLoadNib:(NSWindowController *)aController
{
    [super windowControllerDidLoadNib:aController];
    if (passwordString != nil) {
        [passwordField setStringValue:passwordString];
    }
    if (encryptedData != nil) {
        [informationField setStringValue:@""];
        [self showOpenPasswordSheet];
    }
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSFont *font = [NSFont fontWithName:[defaults stringForKey:EPFontName]
                                   size:[defaults integerForKey:EPPointSize]];
    [textView setFont:font];
}

- (void)showOpenPasswordSheet
{
    [self showOpenPasswordSheetWithInformationText:@""];
}

- (void)showOpenPasswordSheetWithInformationText:informationText
{
    [informationField setStringValue:informationText];
    [NSApp beginSheet:[passwordField window] 
       modalForWindow:[textView window]
        modalDelegate:self
       didEndSelector:@selector(openPasswordSheetDidEnd:returnCode:contextInfo:)
          contextInfo:nil];
}

- (void)openPasswordSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if (!returnCode) {
        [self close];
        return;
    }
    Cryptograph *crypto = [[[Cryptograph alloc] init] autorelease];
    self.passwordString = [passwordField stringValue];
    NSString *decryptedString = [crypto stringDecryptedFromData:encryptedData
                                                   withPassword:passwordString];
    if (decryptedString != nil) {
        [textView setString:decryptedString];
    } else {
        [self showOpenPasswordSheetWithInformationText:@"Password seemed to be incorrect."];
    }
}

- (IBAction)showSavePasswordSheetAndDo:(SEL)actionSelector
{
    [informationField setStringValue:@""];
    [NSApp beginSheet:[passwordField window] 
       modalForWindow:[textView window]
        modalDelegate:self
       didEndSelector:@selector(savePasswordSheetDidEnd:returnCode:contextInfo:)
          contextInfo:actionSelector];
}

- (void)savePasswordSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if (!returnCode) {
        return;
    }
    Cryptograph *crypto = [[[Cryptograph alloc] init] autorelease];
    self.passwordString = [passwordField stringValue];
    self.encryptedData = [crypto dataEncryptedFromString:[textView string]
                                            withPassword:passwordString];
    if (encryptedData != nil) {
        [self performSelector:contextInfo withObject:self];
    } else {
        [self showSavePasswordSheetAndDo:contextInfo];
    }
}

- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError
{
    if (passwordString == nil) {
        return nil;
    }
    Cryptograph *crypto = [[[Cryptograph alloc] init] autorelease];
    NSData *data = [crypto dataEncryptedFromString:[textView string]
                                      withPassword:passwordString];
    self.encryptedData = data;
    return data;
}

- (BOOL)readFromData:(NSData *)data ofType:(NSString *)typeName error:(NSError **)outError
{
    self.encryptedData = data;
    [textView setString:@""];
    return YES;
}

- (IBAction)cancelPasswordSheet:(id)sender
{
    NSWindow *sheet = [sender window];
    [sheet orderOut:sender];
    [NSApp endSheet:sheet returnCode:0];
}

- (IBAction)okPasswordSheet:(id)sender
{
    NSWindow *sheet = [sender window];
    self.passwordString = [passwordField stringValue];
    [sheet orderOut:sender];
    [NSApp endSheet:sheet returnCode:1];
}

- (IBAction)saveDocument:(id)sender
{
    if (passwordString == nil) {
        [self showSavePasswordSheetAndDo:@selector(saveDocument:)];
        return;
    }
    [super saveDocument:sender];
}

- (IBAction)_saveDocumentAs:(id)sender
{
    [super saveDocumentAs:sender];
}

- (IBAction)saveDocumentAs:(id)sender
{
    [self showSavePasswordSheetAndDo:@selector(_saveDocumentAs:)];
}

@end
